#|
	$Id: grobner-tests.lisp,v 1.2 2009/01/19 18:30:17 marek Exp $	
  *--------------------------------------------------------------------------*
  |  Copyright (C) 1994, Marek Rychlik (e-mail: rychlik@math.arizona.edu)    |
  |    Department of Mathematics, University of Arizona, Tucson, AZ 85721    |
  |                                                                          |
  | Everyone is permitted to copy, distribute and modify the code in this    |
  | directory, as long as this copyright note is preserved verbatim.         |
  *--------------------------------------------------------------------------*
|#
;;(use-package '("PARSE" "MONOM" "PRINTER" "COLORED-POLY" "GROBNER" "COEFFICIENT-RING"))

;; The value of this variable is a function that will do evaluation
;; instead of eval; this one just prints what it is evaluating and calls eval

#+kcl
(setf *evalhook* 
  #'(lambda (form env)
      (format t "~&EVAL> ~s~%" form)
      (let (val)
	(time (setf val (multiple-value-list (eval form))))
	(format t "~&VALUES--> ~s~2%" val)
	val)))

(defun run-grobner-test (plist vars pred result &optional (ring *coefficient-ring*))
  (let* ((new-answer (progn (time (string-grobner plist vars
						  :order pred
						  :suppress-value nil
						  :top-reduction-only nil))
			    (terpri)))
	 (answer (rest
		  (string-read-poly
		   result
		   vars :order pred))))
    (terpri)
    (assert (grobner-test answer
			  (rest (string-read-poly plist vars
						  :order pred))
			  pred ring))
    (terpri)
    (assert (set-exclusive-or new-answer answer :test #'tree-equal))))

(dribble "grobner-tests.output")

(warn "Test 1 - Enneper, full reduction")
(run-grobner-test
 "[x-3*u-3*u*v^2+u^3,y-3*v-3*u^2*v+v^3,z-3*u^2+3*v^2]"
 '(u v x y z)
 #'lex>
"[ 54 * V^2 * Y * Z + 27 * V * X^2 - 27 * V * Y^2 + 8 * V * Z^3 + 72 * V * Z^2 - 9 * Y * Z^2 - 27 * Y * Z, 9 * U * X + 6 * V^2 * Z - 9 * V * Y + Z^2 - 9 * Z, 243 * V^2 * X^2 - 243 * V^2 * Y^2 - 1296 * V^2 * Z - 108 * V * Y * Z^2 + 324 * V * Y * Z + 108 * X^2 * Z + 648 * X^2 + 135 * Y^2 * Z - 648 * Y^2 - 4 * Z^4 + 48 * Z^3 + 108 * Z^2 - 1944 * Z, 2 * V^3 + V * Z + 3 * V - Y, 6 * U * V^2 - U * Z + 9 * U - 3 * X, 3 * U^2 - 3 * V^2 - Z, 4 * U * V * Z - 3 * U * Y + 3 * V * X, 9 * U * V * Y - 2 * U * Z^2 + 18 * U * Z - 9 * V^2 * X - 6 * X * Z, 27 * U * Y^2 - 8 * U * Z^3 + 72 * U * Z^2 - 36 * V^2 * X * Z - 27 * V * X * Y - 24 * X * Z^2, 18 * V^2 * Z^2 + 54 * V^2 * Z - 54 * V * Y * Z - 27 * X^2 + 27 * Y^2 + Z^3 - 18 * Z^2 + 81 * Z, 27 * V * X^2 * Z + 81 * V * X^2 + 135 * V * Y^2 * Z - 81 * V * Y^2 + 8 * V * Z^4 + 96 * V * Z^3 + 216 * V * Z^2 + 81 * X^2 * Y - 81 * Y^3 - 12 * Y * Z^3 - 324 * Y * Z, 4374 * V * X^2 * Y + 8748 * V * Y^3 * Z - 4374 * V * Y^3 + 648 * V * Y * Z^4 + 5184 * V * Y * Z^3 + 17496 * V * Y * Z^2 - 729 * X^4 + 5832 * X^2 * Y^2 - 189 * X^2 * Z^3 - 2430 * X^2 * Z^2 + 2187 * X^2 * Z - 5103 * Y^4 - 945 * Y^2 * Z^3 + 972 * Y^2 * Z^2 - 19683 * Y^2 * Z + 8 * Z^6 - 72 * Z^5 - 648 * Z^4 + 5832 * Z^3, 8748 * V * Y^3 * Z^2 + 648 * V * Y * Z^5 + 5832 * V * Y * Z^4 + 17496 * V * Y * Z^3 + 17496 * V * Y * Z^2 - 729 * X^4 * Z - 2187 * X^4 + 5832 * X^2 * Y^2 * Z + 4374 * X^2 * Y^2 - 189 * X^2 * Z^4 - 2997 * X^2 * Z^3 - 5103 * X^2 * Z^2 + 6561 * X^2 * Z - 5103 * Y^4 * Z - 2187 * Y^4 - 945 * Y^2 * Z^4 + 81 * Y^2 * Z^3 - 16767 * Y^2 * Z^2 - 6561 * Y^2 * Z + 8 * Z^7 - 48 * Z^6 - 864 * Z^5 + 3888 * Z^4 + 17496 * Z^3, 2187 * V * X^4 + 69984 * V * X^2 + 8748 * V * Y^4 * Z - 2187 * V * Y^4 + 648 * V * Y^2 * Z^4 + 3240 * V * Y^2 * Z^3 - 11664 * V * Y^2 * Z^2 + 139968 * V * Y^2 * Z - 69984 * V * Y^2 - 192 * V * Z^6 - 3456 * V * Z^5 - 15552 * V * Z^4 + 20736 * V * Z^3 + 186624 * V * Z^2 - 729 * X^4 * Y + 5832 * X^2 * Y^3 - 189 * X^2 * Y * Z^3 - 7047 * X^2 * Y * Z^2 - 23328 * X^2 * Y * Z + 69984 * X^2 * Y - 5103 * Y^5 - 945 * Y^3 * Z^3 + 1215 * Y^3 * Z^2 + 5832 * Y^3 * Z - 69984 * Y^3 + 8 * Y * Z^6 + 288 * Y * Z^5 + 216 * Y * Z^4 + 5184 * Y * Z^3 + 93312 * Y * Z^2 - 279936 * Y * Z, 19683 * X^6 - 59049 * X^4 * Y^2 + 10935 * X^4 * Z^3 + 118098 * X^4 * Z^2 - 59049 * X^4 * Z + 59049 * X^2 * Y^4 + 56862 * X^2 * Y^2 * Z^3 + 118098 * X^2 * Y^2 * Z + 1296 * X^2 * Z^6 + 34992 * X^2 * Z^5 + 174960 * X^2 * Z^4 - 314928 * X^2 * Z^3 - 19683 * Y^6 + 10935 * Y^4 * Z^3 - 118098 * Y^4 * Z^2 - 59049 * Y^4 * Z - 1296 * Y^2 * Z^6 + 34992 * Y^2 * Z^5 - 174960 * Y^2 * Z^4 - 314928 * Y^2 * Z^3 - 64 * Z^9 + 10368 * Z^7 - 419904 * Z^5 ]")

(warn "Test 2 - Cox Example 1")
(run-grobner-test
 "[x^5+y^4+z^3-1, x^3+y^2+z^2-1]"
 '(x y z)
 #'lex>
 "[ X^2 * Z^4 + X^2 * Z^3 - 2 * X^2 * Z^2 - 1/6 * X * Z^10 - 2/3
	       * X * Z^9 - 1/4 * X * Z^8 + 7/6 * X * Z^7 - 1/12 * X * Z^6 - X
	       * Z^5 + X * Z^4 + 1/2 * Y^10 * Z^2 + 1/3 * Y^10 * Z - 5/12 *
	       Y^10 - 1/6 * Y^8 * Z^4 + 1/3 * Y^8 * Z^3 + 1/4 * Y^8 * Z^2 +
	       1/2 * Y^8 + Y^6 * Z^5 - 5/3 * Y^6 * Z^4 - 17/6 * Y^6 * Z^3 +
	       17/6 * Y^6 * Z^2 + 2/3 * Y^6 * Z - 1/3 * Y^6 - 1/3 * Y^4 * Z^7
	       - 7/2 * Y^4 * Z^6 - 3 * Y^4 * Z^5 + 131/12 * Y^4 * Z^4 + 23/4 *
	       Y^4 * Z^3 - 13 * Y^4 * Z^2 - 8/3 * Y^4 * Z + 29/6 * Y^4 - 17/6
	       * Y^2 * Z^8 - 4 * Y^2 * Z^7 + 119/12 * Y^2 * Z^6 + 17/2 * Y^2 *
	       Z^5 - 16 * Y^2 * Z^4 - 14/3 * Y^2 * Z^3 + 12 * Y^2 * Z^2 + 5/3
	       * Y^2 * Z - 55/12 * Y^2 - 7/6 * Z^10 - 7/6 * Z^9 + 55/12 * Z^8
	       + 21/4 * Z^7 - 31/4 * Z^6 - 15/2 * Z^5 + 83/12 * Z^4 + 41/12 *
	       Z^3 - 31/12 * Z^2, X * Y^2 * Z - X * Y^2 - 1/8 * X * Z^10 -
	       7/24 * X * Z^9 + 1/2 * X * Z^8 + 7/8 * X * Z^7 - 23/24 * X *
	       Z^6 + 1/2 * X * Z^4 - 1/2 * X * Z^3 - X * Z + X + 3/8 * Y^10 *
	       Z^2 - 3/8 * Y^10 * Z - 7/24 * Y^10 - 1/8 * Y^8 * Z^4 + 11/24 *
	       Y^8 * Z^3 - 3/8 * Y^8 * Z^2 + 1/4 * Y^8 * Z - 1/4 * Y^8 + 3/4 *
	       Y^6 * Z^5 - 5/2 * Y^6 * Z^4 + 5/6 * Y^6 * Z^3 + 31/12 * Y^6 *
	       Z^2 - Y^6 * Z - 1/3 * Y^6 - 1/4 * Y^4 * Z^7 - 53/24 * Y^4 * Z^6
	       + 11/6 * Y^4 * Z^5 + 113/12 * Y^4 * Z^4 - 55/8 * Y^4 * Z^3 -
	       29/4 * Y^4 * Z^2 + 15/4 * Y^4 * Z + 19/12 * Y^4 - 17/8 * Y^2 *
	       Z^8 + 13/24 * Y^2 * Z^7 + 239/24 * Y^2 * Z^6 - 59/12 * Y^2 *
	       Z^5 - 151/12 * Y^2 * Z^4 + 85/12 * Y^2 * Z^3 + 43/8 * Y^2 * Z^2
	       - 29/8 * Y^2 * Z + 7/24 * Y^2 - 7/8 * Z^10 + 7/12 * Z^9 + 31/8
	       * Z^8 - 11/12 * Z^7 - 103/12 * Z^6 + 17/6 * Z^5 + 127/24 * Z^4
	       - 3/2 * Z^3 - 17/24 * Z^2 + Z - 1, X * Z^11 + 4 * X * Z^10 + X
	       * Z^9 - 10 * X * Z^8 - 4 * X * Z^7 + 8 * X * Z^6 - 3 * Y^10 *
	       Z^3 - 2 * Y^10 * Z^2 + 4 * Y^10 * Z + 4 * Y^10 + Y^8 * Z^5 - 2
	       * Y^8 * Z^4 - 2 * Y^8 * Z^3 - 6 * Y^6 * Z^6 + 10 * Y^6 * Z^5 +
	       20 * Y^6 * Z^4 - 16 * Y^6 * Z^3 - 24 * Y^6 * Z^2 + 8 * Y^6 * Z
	       + 8 * Y^6 + 2 * Y^4 * Z^8 + 21 * Y^4 * Z^7 + 17 * Y^4 * Z^6 -
	       78 * Y^4 * Z^5 - 64 * Y^4 * Z^4 + 90 * Y^4 * Z^3 + 76 * Y^4 *
	       Z^2 - 32 * Y^4 * Z - 32 * Y^4 + 17 * Y^2 * Z^9 + 24 * Y^2 * Z^8
	       - 68 * Y^2 * Z^7 - 80 * Y^2 * Z^6 + 106 * Y^2 * Z^5 + 108 * Y^2
	       * Z^4 - 77 * Y^2 * Z^3 - 70 * Y^2 * Z^2 + 20 * Y^2 * Z + 20 *
	       Y^2 + 7 * Z^11 + 7 * Z^10 - 31 * Z^9 - 42 * Z^8 + 55 * Z^7 + 77
	       * Z^6 - 39 * Z^5 - 62 * Z^4 + 8 * Z^3 + 20 * Z^2, Y^12 - Y^10 +
	       3 * Y^8 * Z^3 - 5 * Y^8 * Z^2 + 2 * Y^8 - 10 * Y^6 * Z^4 + 20 *
	       Y^6 * Z^2 - 10 * Y^6 - 7 * Y^4 * Z^6 + 30 * Y^4 * Z^4 - 6 * Y^4
	       * Z^3 - 30 * Y^4 * Z^2 + 13 * Y^4 - 5 * Y^2 * Z^8 + 20 * Y^2 *
	       Z^6 - 30 * Y^2 * Z^4 + 20 * Y^2 * Z^2 - 5 * Y^2 - Z^10 + Z^9 +
	       5 * Z^8 - 13 * Z^6 + 10 * Z^4 + 3 * Z^3 - 5 * Z^2, X * Y^4 + X
	       * Z^3 - X + Y^4 + 2 * Y^2 * Z^2 - 2 * Y^2 + Z^4 - 2 * Z^2 + 1,
	       X^2 * Y^2 + X^2 * Z^2 - X^2 - Y^4 - Z^3 + 1, X^3 + Y^2 + Z^2 -
	       1 ]")



(warn "Test 3 - Cox Example 2")
(run-grobner-test
 "[x^5+y^4+z^3-1,x^3+y^3+z^2-1]"
 '(x y z)
 #'lex>
 "[ X^3 + Y^3 + Z^2 - 1, X^2 * Y^3 + X^2 * Z^2 - X^2 - Y^4 - Z^3 + 1, 7776 * X^2 * Y * Z - 7776 * X^2 * Y + 7776 * X^2 * Z^6 + 15552 * X^2 * Z^5 + 23328 * X^2 * Z^4 - 23328 * X^2 * Z^3 - 23328 * X^2 * Z^2 - 7776 * X^2 * Z + 7776 * X^2 + 54432 * X * Y^3 * Z - 54432 * X * Y^3 - 93312 * X * Y^2 * Z + 93312 * X * Y^2 - 62093 * X * Z^20 - 481184 * X * Z^19 - 1974401 * X * Z^18 - 4548614 * X * Z^17 - 5402786 * X * Z^16 + 1471927 * X * Z^15 + 15375648 * X * Z^14 + 22142511 * X * Z^13 + 5326449 * X * Z^12 - 20991981 * X * Z^11 - 25192515 * X * Z^10 - 3694290 * X * Z^9 + 10699009 * X * Z^8 + 7091971 * X * Z^7 + 217021 * X * Z^6 + 7776 * X * Z^5 + 9720 * X * Z^4 + 5832 * X * Z^3 + 38880 * X * Z - 38880 * X - 62093 * Y^14 * Z^11 + 450211 * Y^14 * Z^10 + 2449174 * Y^14 * Z^9 + 5152725 * Y^14 * Z^8 + 5893152 * Y^14 * Z^7 + 2999595 * Y^14 * Z^6 - 3276945 * Y^14 * Z^5 - 9750375 * Y^14 * Z^4 - 11437557 * Y^14 * Z^3 - 6849778 * Y^14 * Z^2 - 1891498 * Y^14 * Z - 67438 * Y^14 + 620930 * Y^13 * Z^11 + 2576492 * Y^13 * Z^10 + 5401850 * Y^13 * Z^9 + 7811974 * Y^13 * Z^8 + 8669290 * Y^13 * Z^7 + 5973124 * Y^13 * Z^6 - 1522340 * Y^13 * Z^5 - 9389888 * Y^13 * Z^4 - 11328236 * Y^13 * Z^3 - 6905668 * Y^13 * Z^2 - 1884694 * Y^13 * Z - 78130 * Y^13 + 372558 * Y^12 * Z^12 + 1148500 * Y^12 * Z^11 + 1726276 * Y^12 * Z^10 + 2463088 * Y^12 * Z^9 + 5337046 * Y^12 * Z^8 + 9191446 * Y^12 * Z^7 + 8486896 * Y^12 * Z^6 + 394832 * Y^12 * Z^5 - 9020896 * Y^12 * Z^4 - 11280880 * Y^12 * Z^3 - 6894490 * Y^12 * Z^2 - 1910938 * Y^12 * Z - 55774 * Y^12 - 124186 * Y^11 * Z^13 + 2390654 * Y^11 * Z^12 + 11665603 * Y^11 * Z^11 + 20826185 * Y^11 * Z^10 + 14515838 * Y^11 * Z^9 - 9842841 * Y^11 * Z^8 - 39441258 * Y^11 * Z^7 - 55471863 * Y^11 * Z^6 - 35486271 * Y^11 * Z^5 + 15841297 * Y^11 * Z^4 + 47780995 * Y^11 * Z^3 + 33943290 * Y^11 * Z^2 + 9409862 * Y^11 * Z + 373154 * Y^11 + 62093 * Y^10 * Z^14 + 2654439 * Y^10 * Z^13 + 10433286 * Y^10 * Z^12 + 18068852 * Y^10 * Z^11 + 18157977 * Y^10 * Z^10 + 10384929 * Y^10 * Z^9 - 8576554 * Y^10 * Z^8 - 43983913 * Y^10 * Z^7 - 68351032 * Y^10 * Z^6 - 43934307 * Y^10 * Z^5 + 13952111 * Y^10 * Z^4 + 47175827 * Y^10 * Z^3 + 34193580 * Y^10 * Z^2 + 9408890 * Y^10 * Z + 393566 * Y^10 + 1241860 * Y^9 * Z^14 + 3166008 * Y^9 * Z^13 + 994182 * Y^9 * Z^12 - 1766604 * Y^9 * Z^11 + 10234776 * Y^9 * Z^10 + 30607428 * Y^9 * Z^9 + 19746518 * Y^9 * Z^8 - 35115338 * Y^9 * Z^7 - 78724496 * Y^9 * Z^6 - 53390064 * Y^9 * Z^5 + 12147732 * Y^9 * Z^4 + 46880484 * Y^9 * Z^3 + 34174626 * Y^9 * Z^2 + 9573158 * Y^9 * Z + 248738 * Y^9 - 62093 * Y^8 * Z^15 + 4051605 * Y^8 * Z^14 + 18125525 * Y^8 * Z^13 + 28414036 * Y^8 * Z^12 + 9581124 * Y^8 * Z^11 - 32226896 * Y^8 * Z^10 - 69123395 * Y^8 * Z^9 - 81091480 * Y^8 * Z^8 - 35307862 * Y^8 * Z^7 + 66547639 * Y^8 * Z^6 + 115182683 * Y^8 * Z^5 + 48088910 * Y^8 * Z^4 - 29152980 * Y^8 * Z^3 - 33299528 * Y^8 * Z^2 - 9206714 * Y^8 * Z - 541310 * Y^8 + 124186 * Y^7 * Z^16 + 3818646 * Y^7 * Z^15 + 14285596 * Y^7 * Z^14 + 19298896 * Y^7 * Z^13 + 7117194 * Y^7 * Z^12 - 9270810 * Y^7 * Z^11 - 26804700 * Y^7 * Z^10 - 66254262 * Y^7 * Z^9 - 84309180 * Y^7 * Z^8 - 4494634 * Y^7 * Z^7 + 118530918 * Y^7 * Z^6 + 133783318 * Y^7 * Z^5 + 26287984 * Y^7 * Z^4 - 62322100 * Y^7 * Z^3 - 54131228 * Y^7 * Z^2 - 15072692 * Y^7 * Z - 571580 * Y^7 - 62093 * Y^6 * Z^17 + 1071141 * Y^6 * Z^16 + 1051714 * Y^6 * Z^15 - 10057949 * Y^6 * Z^14 - 21572134 * Y^6 * Z^13 + 6413327 * Y^6 * Z^12 + 59209974 * Y^6 * Z^11 + 39254296 * Y^6 * Z^10 - 78227581 * Y^6 * Z^9 - 143215725 * Y^6 * Z^8 - 32549470 * Y^6 * Z^7 + 132719201 * Y^6 * Z^6 + 148478637 * Y^6 * Z^5 + 29119635 * Y^6 * Z^4 - 61589201 * Y^6 * Z^3 - 54345068 * Y^6 * Z^2 - 15315692 * Y^6 * Z - 359684 * Y^6 + 2732092 * Y^5 * Z^16 + 10988844 * Y^5 * Z^15 + 12430164 * Y^5 * Z^14 - 10283576 * Y^5 * Z^13 - 40089496 * Y^5 * Z^12 - 50953520 * Y^5 * Z^11 - 45553956 * Y^5 * Z^10 - 701820 * Y^5 * Z^9 + 101520364 * Y^5 * Z^8 + 142704156 * Y^5 * Z^7 + 28845308 * Y^5 * Z^6 - 90595828 * Y^5 * Z^5 - 72824464 * Y^5 * Z^4 - 4522320 * Y^5 * Z^3 + 12452276 * Y^5 * Z^2 + 3496256 * Y^5 * Z + 386624 * Y^5 + 62093 * Y^4 * Z^18 + 2157695 * Y^4 * Z^17 + 7080558 * Y^4 * Z^16 + 6495674 * Y^4 * Z^15 - 1846064 * Y^4 * Z^14 + 885080 * Y^4 * Z^13 + 1387642 * Y^4 * Z^12 - 49868373 * Y^4 * Z^11 - 105032993 * Y^4 * Z^10 - 31820374 * Y^4 * Z^9 + 129546847 * Y^4 * Z^8 + 167383594 * Y^4 * Z^7 + 28654796 * Y^4 * Z^6 - 93073209 * Y^4 * Z^5 - 73097103 * Y^4 * Z^4 - 5850743 * Y^4 * Z^3 + 13044224 * Y^4 * Z^2 + 3690656 * Y^4 * Z + 200000 * Y^4 - 124186 * Y^3 * Z^19 - 93066 * Y^3 * Z^18 - 2614317 * Y^3 * Z^17 - 14439323 * Y^3 * Z^16 - 20470190 * Y^3 * Z^15 + 20484593 * Y^3 * Z^14 + 83602226 * Y^3 * Z^13 + 51887995 * Y^3 * Z^12 - 93415586 * Y^3 * Z^11 - 161017882 * Y^3 * Z^10 - 13363721 * Y^3 * Z^9 + 170255391 * Y^3 * Z^8 + 137957690 * Y^3 * Z^7 - 45834331 * Y^3 * Z^6 - 128446523 * Y^3 * Z^5 - 55627499 * Y^3 * Z^4 + 27747047 * Y^3 * Z^3 + 33736442 * Y^3 * Z^2 + 9648974 * Y^3 * Z + 126266 * Y^3 + 620930 * Y^2 * Z^18 + 1955562 * Y^2 * Z^17 - 279292 * Y^2 * Z^16 - 9851406 * Y^2 * Z^15 - 16124282 * Y^2 * Z^14 - 8540932 * Y^2 * Z^13 + 3321430 * Y^2 * Z^12 + 20070502 * Y^2 * Z^11 + 51189864 * Y^2 * Z^10 + 50901220 * Y^2 * Z^9 - 21177336 * Y^2 * Z^8 - 81223016 * Y^2 * Z^7 - 46221546 * Y^2 * Z^6 + 17771150 * Y^2 * Z^5 + 28155166 * Y^2 * Z^4 + 8893592 * Y^2 * Z^3 + 538394 * Y^2 * Z^2 + 93312 * Y^2 * Z - 93312 * Y^2 + 372558 * Y * Z^19 + 775942 * Y * Z^18 - 1285014 * Y * Z^17 - 4633130 * Y * Z^16 - 138226 * Y * Z^15 + 5184446 * Y * Z^14 - 13306902 * Y * Z^13 - 34770050 * Y * Z^12 + 3870366 * Y * Z^11 + 77880544 * Y * Z^10 + 72922124 * Y * Z^9 - 25219024 * Y * Z^8 - 88829334 * Y * Z^7 - 47570630 * Y * Z^6 + 16921524 * Y * Z^5 + 28220776 * Y * Z^4 + 9356264 * Y * Z^3 + 247766 * Y * Z^2 - 7776 * Y * Z + 7776 * Y - 62093 * Z^21 - 294905 * Z^20 - 1896895 * Z^19 - 5672158 * Z^18 - 3792326 * Z^17 + 17358124 * Z^16 + 37860730 * Z^15 + 4109437 * Z^14 - 75897065 * Z^13 - 89966545 * Z^12 + 21700007 * Z^11 + 135785208 * Z^10 + 98509917 * Z^9 - 39012662 * Z^8 - 103259594 * Z^7 - 50374091 * Z^6 + 17195287 * Z^5 + 27941326 * Z^4 + 9673136 * Z^3 + 95162 * Z^2 - 31104 * Z + 31104, 3888 * X^2 * Z^7 + 11664 * X^2 * Z^6 + 19440 * X^2 * Z^5 - 19440 * X^2 * Z^3 - 15552 * X^2 * Z^2 + 69984 * X * Y^3 * Z - 69984 * X * Y^3 - 104976 * X * Y^2 * Z + 104976 * X * Y^2 - 62197 * X * Z^20 - 477679 * X * Z^19 - 1943665 * X * Z^18 - 4414720 * X * Z^17 - 5079481 * X * Z^16 + 1881863 * X * Z^15 + 15324612 * X * Z^14 + 21069324 * X * Z^13 + 3669771 * X * Z^12 - 21506892 * X * Z^11 - 23712288 * X * Z^10 - 1764237 * X * Z^9 + 11052938 * X * Z^8 + 6288353 * X * Z^7 - 343198 * X * Z^6 + 5832 * X * Z^5 + 11664 * X * Z^4 + 34992 * X * Z - 34992 * X - 62197 * Y^14 * Z^11 + 455276 * Y^14 * Z^10 + 2422655 * Y^14 * Z^9 + 4986216 * Y^14 * Z^8 + 5523915 * Y^14 * Z^7 + 2561289 * Y^14 * Z^6 - 3518784 * Y^14 * Z^5 - 9539685 * Y^14 * Z^4 - 10740936 * Y^14 * Z^3 - 6000836 * Y^14 * Z^2 - 1367102 * Y^14 * Z + 74500 * Y^14 + 621970 * Y^13 * Z^11 + 2537698 * Y^13 * Z^10 + 5225662 * Y^13 * Z^9 + 7428554 * Y^13 * Z^8 + 8105042 * Y^13 * Z^7 + 5335154 * Y^13 * Z^6 - 1983340 * Y^13 * Z^5 - 9316858 * Y^13 * Z^4 - 10658716 * Y^13 * Z^3 - 6064016 * Y^13 * Z^2 - 1360298 * Y^13 * Z + 64780 * Y^13 + 373182 * Y^12 * Z^12 + 1124558 * Y^12 * Z^11 + 1645616 * Y^12 * Z^10 + 2338550 * Y^12 * Z^9 + 5165504 * Y^12 * Z^8 + 8820386 * Y^12 * Z^7 + 7823078 * Y^12 * Z^6 - 253010 * Y^12 * Z^5 - 9093788 * Y^12 * Z^4 - 10641134 * Y^12 * Z^3 - 6053324 * Y^12 * Z^2 - 1388486 * Y^12 * Z + 90052 * Y^12 - 124394 * Y^11 * Z^13 + 2403280 * Y^11 * Z^12 + 11520437 * Y^11 * Z^11 + 20025700 * Y^11 * Z^10 + 12994009 * Y^11 * Z^9 - 10993230 * Y^11 * Z^8 - 38847465 * Y^11 * Z^7 - 52680669 * Y^11 * Z^6 - 31408200 * Y^11 * Z^5 + 18625895 * Y^11 * Z^4 + 46816940 * Y^11 * Z^3 + 30408756 * Y^11 * Z^2 + 6785938 * Y^11 * Z - 331676 * Y^11 + 62197 * Y^10 * Z^14 + 2654574 * Y^10 * Z^13 + 10265835 * Y^10 * Z^12 + 17348077 * Y^10 * Z^11 + 16847943 * Y^10 * Z^10 + 9032604 * Y^10 * Z^9 - 9390554 * Y^10 * Z^8 - 43483352 * Y^10 * Z^7 - 65292989 * Y^10 * Z^6 - 38911488 * Y^10 * Z^5 + 17408485 * Y^10 * Z^4 + 46335370 * Y^10 * Z^3 + 30703272 * Y^10 * Z^2 + 6786910 * Y^10 * Z - 318068 * Y^10 + 1243940 * Y^9 * Z^14 + 3085092 * Y^9 * Z^13 + 763326 * Y^9 * Z^12 - 1861182 * Y^9 * Z^11 + 10386480 * Y^9 * Z^10 + 29961138 * Y^9 * Z^9 + 17534260 * Y^9 * Z^8 - 36765466 * Y^9 * Z^7 - 76413538 * Y^9 * Z^6 - 47606934 * Y^9 * Z^5 + 16339548 * Y^9 * Z^4 + 46176990 * Y^9 * Z^3 + 30698412 * Y^9 * Z^2 + 6962842 * Y^9 * Z - 479420 * Y^9 - 62197 * Y^8 * Z^15 + 4062702 * Y^8 * Z^14 + 17875228 * Y^8 * Z^13 + 27161252 * Y^8 * Z^12 + 7471074 * Y^8 * Z^11 - 33102991 * Y^8 * Z^10 - 66936577 * Y^8 * Z^9 - 76120232 * Y^8 * Z^8 - 29301146 * Y^8 * Z^7 + 69509327 * Y^8 * Z^6 + 110691688 * Y^8 * Z^5 + 39460090 * Y^8 * Z^4 - 33130656 * Y^8 * Z^3 - 31174168 * Y^8 * Z^2 - 6567238 * Y^8 * Z + 139220 * Y^8 + 124394 * Y^7 * Z^16 + 3816420 * Y^7 * Z^15 + 14043134 * Y^7 * Z^14 + 18301298 * Y^7 * Z^13 + 5672886 * Y^7 * Z^12 - 9874692 * Y^7 * Z^11 - 26175312 * Y^7 * Z^10 - 64418844 * Y^7 * Z^9 - 79656150 * Y^7 * Z^8 + 1847500 * Y^7 * Z^7 + 119457606 * Y^7 * Z^6 + 125367140 * Y^7 * Z^5 + 16066196 * Y^7 * Z^4 - 64709996 * Y^7 * Z^3 - 49525624 * Y^7 * Z^2 - 10879468 * Y^7 * Z + 559064 * Y^7 - 62197 * Y^6 * Z^17 + 1077246 * Y^6 * Z^16 + 979745 * Y^6 * Z^15 - 10159270 * Y^6 * Z^14 - 20912123 * Y^6 * Z^13 + 8030785 * Y^6 * Z^12 + 58997319 * Y^6 * Z^11 + 35006123 * Y^6 * Z^10 - 81555203 * Y^6 * Z^9 - 137987784 * Y^6 * Z^8 - 21773417 * Y^6 * Z^7 + 135926551 * Y^6 * Z^6 + 139019436 * Y^6 * Z^5 + 17749911 * Y^6 * Z^4 - 64170334 * Y^6 * Z^3 - 49799728 * Y^6 * Z^2 - 11139964 * Y^6 * Z + 796232 * Y^6 + 2736668 * Y^5 * Z^16 + 10817568 * Y^5 * Z^15 + 11660088 * Y^5 * Z^14 - 11254696 * Y^5 * Z^13 - 39489536 * Y^5 * Z^12 - 48089980 * Y^5 * Z^11 - 41808924 * Y^5 * Z^10 + 2708652 * Y^5 * Z^9 + 101941496 * Y^5 * Z^8 + 135684168 * Y^5 * Z^7 + 18081196 * Y^5 * Z^6 - 93428348 * Y^5 * Z^5 - 66181556 * Y^5 * Z^4 + 1268748 * Y^5 * Z^3 + 12830488 * Y^5 * Z^2 + 2410528 * Y^5 * Z + 148432 * Y^5 + 62197 * Y^4 * Z^18 + 2156998 * Y^4 * Z^17 + 6941979 * Y^4 * Z^16 + 5993371 * Y^4 * Z^15 - 2355712 * Y^4 * Z^14 + 998824 * Y^4 * Z^13 + 1373297 * Y^4 * Z^12 - 50094117 * Y^4 * Z^11 - 101736412 * Y^4 * Z^10 - 24063602 * Y^4 * Z^9 + 132624275 * Y^4 * Z^8 + 158340584 * Y^4 * Z^7 + 15863623 * Y^4 * Z^6 - 95842332 * Y^4 * Z^5 - 66222615 * Y^4 * Z^4 - 131590 * Y^4 * Z^3 + 13526440 * Y^4 * Z^2 + 2641864 * Y^4 * Z - 77072 * Y^4 - 124394 * Y^3 * Z^19 - 84600 * Y^3 * Z^18 - 2610963 * Y^3 * Z^17 - 14282032 * Y^3 * Z^16 - 19475671 * Y^3 * Z^15 + 22068016 * Y^3 * Z^14 + 82420621 * Y^3 * Z^13 + 45846113 * Y^3 * Z^12 - 97789063 * Y^3 * Z^11 - 154747355 * Y^3 * Z^10 - 1148833 * Y^3 * Z^9 + 172225566 * Y^3 * Z^8 + 125604427 * Y^3 * Z^7 - 56776217 * Y^3 * Z^6 - 125597464 * Y^3 * Z^5 - 45915757 * Y^3 * Z^4 + 32235268 * Y^3 * Z^3 + 31703908 * Y^3 * Z^2 + 7044490 * Y^3 * Z - 596060 * Y^3 + 621970 * Y^2 * Z^18 + 1915728 * Y^2 * Z^17 - 421886 * Y^2 * Z^16 - 9863628 * Y^2 * Z^15 - 15450484 * Y^2 * Z^14 - 7344470 * Y^2 * Z^13 + 4006574 * Y^2 * Z^12 + 19868540 * Y^2 * Z^11 + 49836054 * Y^2 * Z^10 + 47268530 * Y^2 * Z^9 - 25125354 * Y^2 * Z^8 - 80065726 * Y^2 * Z^7 - 40208808 * Y^2 * Z^6 + 21532348 * Y^2 * Z^5 + 26910314 * Y^2 * Z^4 + 6653686 * Y^2 * Z^3 - 133388 * Y^2 * Z^2 + 104976 * Y^2 * Z - 104976 * Y^2 + 373182 * Y * Z^19 + 751376 * Y * Z^18 - 1344852 * Y * Z^17 - 4556674 * Y * Z^16 + 201616 * Y * Z^15 + 5236534 * Y * Z^14 - 13720314 * Y * Z^13 - 33940378 * Y * Z^12 + 6467994 * Y * Z^11 + 77962850 * Y * Z^10 + 67373776 * Y * Z^9 - 30962138 * Y * Z^8 - 87379542 * Y * Z^7 - 40930240 * Y * Z^6 + 20745570 * Y * Z^5 + 27005570 * Y * Z^4 + 7185370 * Y * Z^3 - 469700 * Y * Z^2 - 62197 * Z^21 - 291088 * Z^20 - 1878962 * Z^19 - 5547434 * Z^18 - 3387640 * Z^17 + 17694533 * Z^16 + 36718052 * Z^15 + 1299572 * Z^14 - 76544146 * Z^13 - 84649877 * Z^12 + 28651966 * Z^11 + 134827431 * Z^10 + 88593549 * Z^9 - 46839364 * Z^8 - 100799218 * Z^7 - 42613627 * Z^6 + 21217046 * Z^5 + 26735354 * Z^4 + 7526542 * Z^3 - 650492 * Z^2 - 34992 * Z + 34992, X * Y^4 + X * Z^3 - X + Y^6 + 2 * Y^3 * Z^2 - 2 * Y^3 + Z^4 - 2 * Z^2 + 1, 69984 * X * Y * Z^2 - 139968 * X * Y * Z + 69984 * X * Y - 44425 * X * Z^20 - 327070 * X * Z^19 - 1278214 * X * Z^18 - 2698855 * X * Z^17 - 2572042 * X * Z^16 + 2619449 * X * Z^15 + 10650408 * X * Z^14 + 11493837 * X * Z^13 - 2616810 * X * Z^12 - 16744470 * X * Z^11 - 12049440 * X * Z^10 + 4801407 * X * Z^9 + 8854766 * X * Z^8 + 1892306 * X * Z^7 - 1980847 * X * Z^6 + 17496 * X * Z^5 - 34992 * X * Z^4 + 17496 * X * Z^3 - 69984 * X * Z^2 + 139968 * X * Z - 69984 * X - 44425 * Y^14 * Z^11 + 339305 * Y^14 * Z^10 + 1628711 * Y^14 * Z^9 + 3000105 * Y^14 * Z^8 + 2743338 * Y^14 * Z^7 + 437586 * Y^14 * Z^6 - 3228714 * Y^14 * Z^5 - 6067476 * Y^14 * Z^4 - 5393667 * Y^14 * Z^3 - 1569800 * Y^14 * Z^2 + 632812 * Y^14 * Z + 501226 * Y^14 + 444250 * Y^13 * Z^11 + 1671400 * Y^13 * Z^10 + 3140020 * Y^13 * Z^9 + 4046756 * Y^13 * Z^8 + 3960482 * Y^13 * Z^7 + 1782242 * Y^13 * Z^6 - 2836264 * Y^13 * Z^5 - 6324094 * Y^13 * Z^4 - 5404156 * Y^13 * Z^3 - 1661654 * Y^13 * Z^2 + 685300 * Y^13 * Z + 474982 * Y^13 + 266550 * Y^12 * Z^12 + 718520 * Y^12 * Z^11 + 910274 * Y^12 * Z^10 + 1263518 * Y^12 * Z^9 + 3116408 * Y^12 * Z^8 + 5071808 * Y^12 * Z^7 + 3440132 * Y^12 * Z^6 - 2194496 * Y^12 * Z^5 - 6620078 * Y^12 * Z^4 - 5499938 * Y^12 * Z^3 - 1622288 * Y^12 * Z^2 + 632812 * Y^12 * Z + 501226 * Y^12 - 88850 * Y^11 * Z^13 + 1744810 * Y^11 * Z^12 + 7686377 * Y^11 * Z^11 + 11626231 * Y^11 * Z^10 + 4404103 * Y^11 * Z^9 - 11359143 * Y^11 * Z^8 - 25510554 * Y^11 * Z^7 - 28425462 * Y^11 * Z^6 - 9426528 * Y^11 * Z^5 + 21880364 * Y^11 * Z^4 + 29810057 * Y^11 * Z^3 + 10350756 * Y^11 * Z^2 - 3181556 * Y^11 * Z - 2497382 * Y^11 + 44425 * Y^10 * Z^14 + 1881945 * Y^10 * Z^13 + 6728289 * Y^10 * Z^12 + 9990070 * Y^10 * Z^11 + 7801347 * Y^10 * Z^10 + 2153415 * Y^10 * Z^9 - 9140507 * Y^10 * Z^8 - 29133398 * Y^10 * Z^7 - 36482669 * Y^10 * Z^6 - 11662110 * Y^10 * Z^5 + 22936006 * Y^10 * Z^4 + 29956543 * Y^10 * Z^3 + 10775034 * Y^10 * Z^2 - 3478988 * Y^10 * Z - 2348666 * Y^10 + 888500 * Y^9 * Z^14 + 1921200 * Y^9 * Z^13 - 187950 * Y^9 * Z^12 - 1596036 * Y^9 * Z^11 + 7831314 * Y^9 * Z^10 + 19135710 * Y^9 * Z^9 + 5385232 * Y^9 * Z^8 - 31118752 * Y^9 * Z^7 - 46554784 * Y^9 * Z^6 - 15507324 * Y^9 * Z^5 + 24599634 * Y^9 * Z^4 + 30284550 * Y^9 * Z^3 + 10578204 * Y^9 * Z^2 - 3111572 * Y^9 * Z - 2532374 * Y^9 - 44425 * Y^8 * Z^15 + 2915955 * Y^8 * Z^14 + 11847046 * Y^8 * Z^13 + 15235952 * Y^8 * Z^12 - 1342215 * Y^8 * Z^11 - 26096155 * Y^8 * Z^10 - 40336258 * Y^8 * Z^9 - 38148698 * Y^8 * Z^8 - 1947773 * Y^8 * Z^7 + 58365386 * Y^8 * Z^6 + 63744949 * Y^8 * Z^5 + 662692 * Y^8 * Z^4 - 35221464 * Y^8 * Z^3 - 15349894 * Y^8 * Z^2 + 3269036 * Y^8 * Z + 2453642 * Y^8 + 88850 * Y^7 * Z^16 + 2697690 * Y^7 * Z^15 + 9160898 * Y^7 * Z^14 + 9782204 * Y^7 * Z^13 - 505866 * Y^7 * Z^12 - 8841150 * Y^7 * Z^11 - 16474014 * Y^7 * Z^10 - 39769140 * Y^7 * Z^9 - 41664378 * Y^7 * Z^8 + 21330064 * Y^7 * Z^7 + 86946000 * Y^7 * Z^6 + 61819454 * Y^7 * Z^5 - 20296984 * Y^7 * Z^4 - 53124800 * Y^7 * Z^3 - 20474824 * Y^7 * Z^2 + 5569880 * Y^7 * Z + 3756116 * Y^7 - 44425 * Y^6 * Z^17 + 783555 * Y^6 * Z^16 + 456911 * Y^6 * Z^15 - 7506655 * Y^6 * Z^14 - 12668822 * Y^6 * Z^13 + 10778914 * Y^6 * Z^12 + 40955979 * Y^6 * Z^11 + 11243399 * Y^6 * Z^10 - 68036522 * Y^6 * Z^9 - 80668677 * Y^6 * Z^8 + 18392944 * Y^6 * Z^7 + 105699982 * Y^6 * Z^6 + 68420946 * Y^6 * Z^5 - 23355948 * Y^6 * Z^4 - 52987825 * Y^6 * Z^3 - 20457328 * Y^6 * Z^2 + 4905032 * Y^6 * Z + 4088540 * Y^6 + 1954700 * Y^5 * Z^16 + 7105380 * Y^5 * Z^15 + 5800536 * Y^5 * Z^14 - 10997536 * Y^5 * Z^13 - 25975004 * Y^5 * Z^12 - 24957052 * Y^5 * Z^11 - 17808216 * Y^5 * Z^10 + 12529572 * Y^5 * Z^9 + 73310048 * Y^5 * Z^8 + 73580532 * Y^5 * Z^7 - 21170564 * Y^5 * Z^6 - 74140892 * Y^5 * Z^5 - 25675184 * Y^5 * Z^4 + 18268092 * Y^5 * Z^3 + 10504900 * Y^5 * Z^2 - 1388096 * Y^5 * Z - 941216 * Y^5 + 44425 * Y^4 * Z^18 + 1526545 * Y^4 * Z^17 + 4467129 * Y^4 * Z^16 + 2647468 * Y^4 * Z^15 - 3245086 * Y^4 * Z^14 + 1090138 * Y^4 * Z^13 + 899768 * Y^4 * Z^12 - 36144372 * Y^4 * Z^11 - 61462426 * Y^4 * Z^10 + 7525711 * Y^4 * Z^9 + 103064435 * Y^4 * Z^8 + 82980083 * Y^4 * Z^7 - 28598633 * Y^4 * Z^6 - 75995106 * Y^4 * Z^5 - 25336908 * Y^4 * Z^4 + 18247517 * Y^4 * Z^3 + 10793584 * Y^4 * Z^2 - 1738016 * Y^4 * Z - 766256 * Y^4 - 88850 * Y^3 * Z^19 - 32190 * Y^3 * Z^18 - 1842423 * Y^3 * Z^17 - 9605029 * Y^3 * Z^16 - 10602997 * Y^3 * Z^15 + 20587543 * Y^3 * Z^14 + 54430174 * Y^3 * Z^13 + 13274582 * Y^3 * Z^12 - 82735585 * Y^3 * Z^11 - 89174777 * Y^3 * Z^10 + 37559804 * Y^3 * Z^9 + 127272309 * Y^3 * Z^8 + 49867042 * Y^3 * Z^7 - 74430398 * Y^3 * Z^6 - 79475920 * Y^3 * Z^5 - 1372204 * Y^3 * Z^4 + 36291181 * Y^3 * Z^3 + 15564220 * Y^3 * Z^2 - 2796644 * Y^3 * Z - 2689838 * Y^3 + 444250 * Y^2 * Z^18 + 1227150 * Y^2 * Z^17 - 752630 * Y^2 * Z^16 - 7006014 * Y^2 * Z^15 - 8783974 * Y^2 * Z^14 - 1436450 * Y^2 * Z^13 + 4950830 * Y^2 * Z^12 + 13378064 * Y^2 * Z^11 + 30957924 * Y^2 * Z^10 + 21952328 * Y^2 * Z^9 - 30170622 * Y^2 * Z^8 - 52627432 * Y^2 * Z^7 - 9384858 * Y^2 * Z^6 + 26513908 * Y^2 * Z^5 + 15113990 * Y^2 * Z^4 - 2001554 * Y^2 * Z^3 - 2374910 * Y^2 * Z^2 + 266550 * Y * Z^19 + 451970 * Y * Z^18 - 1140996 * Y * Z^17 - 2972806 * Y * Z^16 + 1218640 * Y * Z^15 + 3837574 * Y * Z^14 - 11018460 * Y * Z^13 - 21334144 * Y * Z^12 + 12799086 * Y * Z^11 + 55234262 * Y * Z^10 + 29962348 * Y * Z^9 - 39724286 * Y * Z^8 - 56881602 * Y * Z^7 - 8218528 * Y * Z^6 + 26664768 * Y * Z^5 + 15232088 * Y * Z^4 - 2010302 * Y * Z^3 - 2436146 * Y * Z^2 + 139968 * Y * Z - 69984 * Y - 44425 * Z^21 - 193795 * Z^20 - 1274354 * Z^19 - 3527528 * Z^18 - 1109995 * Z^17 + 13565093 * Z^16 + 22302653 * Z^15 - 7954033 * Z^14 - 56030677 * Z^13 - 42908546 * Z^12 + 42041860 * Z^11 + 91874913 * Z^10 + 31327296 * Z^9 - 57432520 * Z^8 - 63266461 * Z^7 - 5375344 * Z^6 + 27268337 * Z^5 + 14641598 * Z^4 - 1214234 * Z^3 - 2619854 * Z^2 - 139968 * Z + 69984, X * Z^21 + 9 * X * Z^20 + 42 * X * Z^19 + 117 * X * Z^18 + 195 * X * Z^17 + 123 * X * Z^16 - 232 * X * Z^15 - 678 * X * Z^14 - 660 * X * Z^13 + 45 * X * Z^12 + 783 * X * Z^11 + 744 * X * Z^10 + 115 * X * Z^9 - 300 * X * Z^8 - 240 * X * Z^7 - 64 * X * Z^6 + Y^14 * Z^12 - 6 * Y^14 * Z^11 - 48 * Y^14 * Z^10 - 136 * Y^14 * Z^9 - 219 * Y^14 * Z^8 - 210 * Y^14 * Z^7 - 57 * Y^14 * Z^6 + 198 * Y^14 * Z^5 + 408 * Y^14 * Z^4 + 422 * Y^14 * Z^3 + 264 * Y^14 * Z^2 + 96 * Y^14 * Z + 16 * Y^14 - 10 * Y^13 * Z^12 - 54 * Y^13 * Z^11 - 144 * Y^13 * Z^10 - 256 * Y^13 * Z^9 - 342 * Y^13 * Z^8 - 336 * Y^13 * Z^7 - 168 * Y^13 * Z^6 + 132 * Y^13 * Z^5 + 384 * Y^13 * Z^4 + 418 * Y^13 * Z^3 + 264 * Y^13 * Z^2 + 96 * Y^13 * Z + 16 * Y^13 - 6 * Y^12 * Z^13 - 26 * Y^12 * Z^12 - 54 * Y^12 * Z^11 - 84 * Y^12 * Z^10 - 150 * Y^12 * Z^9 - 276 * Y^12 * Z^8 - 366 * Y^12 * Z^7 - 254 * Y^12 * Z^6 + 66 * Y^12 * Z^5 + 360 * Y^12 * Z^4 + 414 * Y^12 * Z^3 + 264 * Y^12 * Z^2 + 96 * Y^12 * Z + 16 * Y^12 + 2 * Y^11 * Z^14 - 36 * Y^11 * Z^13 - 235 * Y^11 * Z^12 - 590 * Y^11 * Z^11 - 750 * Y^11 * Z^10 - 308 * Y^11 * Z^9 + 711 * Y^11 * Z^8 + 1770 * Y^11 * Z^7 + 2019 * Y^11 * Z^6 + 922 * Y^11 * Z^5 - 792 * Y^11 * Z^4 - 1642 * Y^11 * Z^3 - 1240 * Y^11 * Z^2 - 480 * Y^11 * Z - 80 * Y^11 - Y^10 * Z^15 - 44 * Y^10 * Z^14 - 222 * Y^10 * Z^13 - 523 * Y^10 * Z^12 - 743 * Y^10 * Z^11 - 684 * Y^10 * Z^10 - 223 * Y^10 * Z^9 + 795 * Y^10 * Z^8 + 2058 * Y^10 * Z^7 + 2449 * Y^10 * Z^6 + 1232 * Y^10 * Z^5 - 672 * Y^10 * Z^4 - 1622 * Y^10 * Z^3 - 1240 * Y^10 * Z^2 - 480 * Y^10 * Z - 80 * Y^10 - 20 * Y^9 * Z^15 - 76 * Y^9 * Z^14 - 90 * Y^9 * Z^13 - 18 * Y^9 * Z^12 - 138 * Y^9 * Z^11 - 684 * Y^9 * Z^10 - 1018 * Y^9 * Z^9 - 88 * Y^9 * Z^8 + 1806 * Y^9 * Z^7 + 2738 * Y^9 * Z^6 + 1542 * Y^9 * Z^5 - 552 * Y^9 * Z^4 - 1602 * Y^9 * Z^3 - 1240 * Y^9 * Z^2 - 480 * Y^9 * Z - 80 * Y^9 + Y^8 * Z^16 - 64 * Y^8 * Z^15 - 373 * Y^8 * Z^14 - 856 * Y^8 * Z^13 - 877 * Y^8 * Z^12 + 88 * Y^8 * Z^11 + 1680 * Y^8 * Z^10 + 2968 * Y^8 * Z^9 + 2781 * Y^8 * Z^8 + 314 * Y^8 * Z^7 - 2900 * Y^8 * Z^6 - 3648 * Y^8 * Z^5 - 1472 * Y^8 * Z^4 + 718 * Y^8 * Z^3 + 1080 * Y^8 * Z^2 + 480 * Y^8 * Z + 80 * Y^8 - 2 * Y^7 * Z^17 - 64 * Y^7 * Z^16 - 308 * Y^7 * Z^15 - 630 * Y^7 * Z^14 - 622 * Y^7 * Z^13 - 156 * Y^7 * Z^12 + 558 * Y^7 * Z^11 + 1686 * Y^7 * Z^10 + 2916 * Y^7 * Z^9 + 2318 * Y^7 * Z^8 - 1112 * Y^7 * Z^7 - 4496 * Y^7 * Z^6 - 4116 * Y^7 * Z^5 - 656 * Y^7 * Z^4 + 1916 * Y^7 * Z^3 + 1872 * Y^7 * Z^2 + 768 * Y^7 * Z + 128 * Y^7 + Y^6 * Z^18 - 16 * Y^6 * Z^17 - 38 * Y^6 * Z^16 + 132 * Y^6 * Z^15 + 541 * Y^6 * Z^14 + 414 * Y^6 * Z^13 - 902 * Y^6 * Z^12 - 1874 * Y^6 * Z^11 - 26 * Y^6 * Z^10 + 3544 * Y^6 * Z^9 + 4061 * Y^6 * Z^8 - 270 * Y^6 * Z^7 - 4787 * Y^6 * Z^6 - 4584 * Y^6 * Z^5 - 848 * Y^6 * Z^4 + 1884 * Y^6 * Z^3 + 1872 * Y^6 * Z^2 + 768 * Y^6 * Z + 128 * Y^6 - 44 * Y^5 * Z^17 - 232 * Y^5 * Z^16 - 444 * Y^5 * Z^15 - 176 * Y^5 * Z^14 + 748 * Y^5 * Z^13 + 1712 * Y^5 * Z^12 + 2096 * Y^5 * Z^11 + 1356 * Y^5 * Z^10 - 1244 * Y^5 * Z^9 - 4336 * Y^5 * Z^8 - 4180 * Y^5 * Z^7 - 324 * Y^5 * Z^6 + 2748 * Y^5 * Z^5 + 2320 * Y^5 * Z^4 + 512 * Y^5 * Z^3 - 288 * Y^5 * Z^2 - 192 * Y^5 * Z - 32 * Y^5 - Y^4 * Z^19 - 36 * Y^4 * Z^18 - 158 * Y^4 * Z^17 - 265 * Y^4 * Z^16 - 160 * Y^4 * Z^15 - 32 * Y^4 * Z^14 - 25 * Y^4 * Z^13 + 770 * Y^4 * Z^12 + 2683 * Y^4 * Z^11 + 3037 * Y^4 * Z^10 - 565 * Y^4 * Z^9 - 5025 * Y^4 * Z^8 - 4922 * Y^4 * Z^7 - 497 * Y^4 * Z^6 + 2820 * Y^4 * Z^5 + 2368 * Y^4 * Z^4 + 520 * Y^4 * Z^3 - 288 * Y^4 * Z^2 - 192 * Y^4 * Z - 32 * Y^4 + 2 * Y^3 * Z^20 + 4 * Y^3 * Z^19 + 45 * Y^3 * Z^18 + 286 * Y^3 * Z^17 + 642 * Y^3 * Z^16 + 202 * Y^3 * Z^15 - 1587 * Y^3 * Z^14 - 2686 * Y^3 * Z^13 - 240 * Y^3 * Z^12 + 4030 * Y^3 * Z^11 + 4238 * Y^3 * Z^10 - 1108 * Y^3 * Z^9 - 5533 * Y^3 * Z^8 - 3486 * Y^3 * Z^7 + 1823 * Y^3 * Z^6 + 3882 * Y^3 * Z^5 + 1768 * Y^3 * Z^4 - 642 * Y^3 * Z^3 - 1080 * Y^3 * Z^2 - 480 * Y^3 * Z - 80 * Y^3 - 10 * Y^2 * Z^19 - 44 * Y^2 * Z^18 - 40 * Y^2 * Z^17 + 148 * Y^2 * Z^16 + 460 * Y^2 * Z^15 + 544 * Y^2 * Z^14 + 254 * Y^2 * Z^13 - 318 * Y^2 * Z^12 - 1258 * Y^2 * Z^11 - 2018 * Y^2 * Z^10 - 1108 * Y^2 * Z^9 + 1306 * Y^2 * Z^8 + 2552 * Y^2 * Z^7 + 1334 * Y^2 * Z^6 - 418 * Y^2 * Z^5 - 872 * Y^2 * Z^4 - 432 * Y^2 * Z^3 - 80 * Y^2 * Z^2 - 6 * Y * Z^20 - 20 * Y * Z^19 + 2 * Y * Z^18 + 94 * Y * Z^17 + 106 * Y * Z^16 - 42 * Y * Z^15 + 112 * Y * Z^14 + 784 * Y * Z^13 + 746 * Y * Z^12 - 1040 * Y * Z^11 - 2768 * Y * Z^10 - 1716 * Y * Z^9 + 1318 * Y * Z^8 + 2770 * Y * Z^7 + 1442 * Y * Z^6 - 398 * Y * Z^5 - 872 * Y * Z^4 - 432 * Y * Z^3 - 80 * Y * Z^2 + Z^22 + 6 * Z^21 + 37 * Z^20 + 132 * Z^19 + 191 * Z^18 - 156 * Z^17 - 927 * Z^16 - 972 * Z^15 + 822 * Z^14 + 2938 * Z^13 + 2097 * Z^12 - 1862 * Z^11 - 4500 * Z^10 - 2504 * Z^9 + 1617 * Z^8 + 3228 * Z^7 + 1614 * Z^6 - 378 * Z^5 - 872 * Z^4 - 432 * Z^3 - 80 * Z^2, Y^15 + 5 * Y^12 * Z^2 - 6 * Y^12 + 10 * Y^9 * Z^4 - 20 * Y^9 * Z^2 + 10 * Y^9 - 3 * Y^8 * Z^3 + 3 * Y^8 + 10 * Y^6 * Z^6 - 30 * Y^6 * Z^4 + 30 * Y^6 * Z^2 - 10 * Y^6 - 3 * Y^4 * Z^6 + 6 * Y^4 * Z^3 - 3 * Y^4 + 5 * Y^3 * Z^8 - 20 * Y^3 * Z^6 + 30 * Y^3 * Z^4 - 20 * Y^3 * Z^2 + 5 * Y^3 + Z^10 - Z^9 - 5 * Z^8 + 13 * Z^6 - 10 * Z^4 - 3 * Z^3 + 5 * Z^2 ]")

(dribble)
#+kcl(setf *evalhook* nil)
